package pong;

import java.awt.Point;
import java.io.Serializable;

import pong.client.Paddle;

/**
 * Implements the state of the game.
 * This includes all players, their state, and the ball.
 * This is serializable, so it can be passed through the network in an ExtendedMessage
 * (see {@link appia.message.ExtendedMessage})
 * @author mocito
 */
public class GameState implements Serializable {

	private static final long serialVersionUID = -8670003468677857471L;
	/**
	 * Maximum number of players.
	 */
	public static final int MAX_PLAYERS = 4;

	private BallState ballState;
	private Point[] playerPosition = new Point[MAX_PLAYERS];
	private int[] playerScore = new int[MAX_PLAYERS];
	private boolean[] activePlayer = new boolean[MAX_PLAYERS];

	/**
	 * Class Constructor. 
	 * @param ballState the state of the ball.
	 */
	public GameState(BallState ballState) {
		super();
		this.ballState = ballState;
		playerPosition[0] = new Point(10,(GameArea.HEIGHT-GameArea.X)/2 - (Paddle.LAR/2)); // Paddle vertical esquerdo
		playerPosition[1] = new Point(GameArea.WIDTH-Paddle.ALT-10,(GameArea.HEIGHT-GameArea.X)/2 - (Paddle.LAR/2)); // Paddle vertical direito
		playerPosition[2] = new Point((GameArea.WIDTH/2) - (Paddle.LAR/2),10); // Paddle horizontal topo
		playerPosition[3] = new Point((GameArea.WIDTH/2) - (Paddle.LAR/2),GameArea.HEIGHT-Paddle.ALT-10); // Paddle horizontal fundo
	}
	
	/**
	 * Resets one player. Set the player paddle to the middle of the wall;
	 * Set the pontuation to 0; sets active player to false.
	 * @param player the player to reset.
	 */
	public void resetPlayer(int player) {
		switch (player) {
		case 0:
			playerPosition[player] = new Point(10,(GameArea.HEIGHT-GameArea.X)/2 - (Paddle.LAR/2)); // Paddle vertical esquerdo
			break;
		case 1:
			playerPosition[player] = new Point(GameArea.WIDTH-Paddle.ALT-10,(GameArea.HEIGHT-GameArea.X)/2 - (Paddle.LAR/2)); // Paddle vertical direito
			break;
		case 2:
			playerPosition[player] = new Point((GameArea.WIDTH/2) - (Paddle.LAR/2),10); // Paddle horizontal topo
			break;
		case 3:
			playerPosition[player] = new Point((GameArea.WIDTH/2) - (Paddle.LAR/2),GameArea.HEIGHT-Paddle.ALT-10); // Paddle horizontal fundo
			break;
		}
		playerScore[player] = 0;
		activePlayer[player] = false;
	}
	
	/**
	 * Gets the ball state.
	 * @return the ball state.
	 */
	public BallState getBallState() {
		return ballState;
	}
	
	/**
	 * set the ball state.
	 * @param ballState the new state of the ball.
	 */
	public void setBallState(BallState ballState) {
		this.ballState = ballState;
	}

	/**
	 * gets the player position.
	 * @param player the player to query the position.
	 * @return the position of the requested player.
	 */
	public Point getPlayerPosition(int player){
		return playerPosition[player];
	}
	
	/**
	 * Sets the player position
	 * @param player the player to set.
	 * @param pos the new position of the player.
	 */
	public void setPlayerPosition(int player, Point pos){
		playerPosition[player] = pos;
	}
	
	/**
	 * Gets the player Score.
	 * @param player the player to query the score.
	 * @return the score of the player.
	 */
	public int getPlayerScore(int player){
		return playerScore[player];
	}
	
	/**
	 * Sets the player score.
	 * @param player the player to update.
	 * @param score the new score of the player.
	 */
	public void setPlayerScore(int player, int score){
		playerScore[player] = score;
	}
	
	/**
	 * Increments the Player score.
	 * @param player the player to update.
	 */
	public void incPlayerScore(int player) {
		if (activePlayer[player])
			playerScore[player]++;
	}
	
	/**
	 * Sets the player to active. This is used when the player press the
	 * Start button (becoming active on the game).
	 * @param player the player to set
	 */
	public void setActivePlayer(int player) {
		activePlayer[player] = true;
	}
	
	/**
	 * Sets the player to inactive. This is used when the player process dies (becoming
	 * inactive on the game).
	 * @param player the player to set
	 */
	public void setInactivePlayer(int player) {
		activePlayer[player] = false;
	}
	
	/**
	 * returns an array with the values of active/inactive values for each player.
	 * @return an array of booleans with the values of active/inactive values for each player.
	 */
	public boolean[] getActivePlayers() {
		return activePlayer;
	}
}
