/*
 * Created on 09-Sep-2003
 *
 */
package pong.client;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;

/**
 * Class that represents one paddle of the game.
 * This class is created automatically by the API provided (see @link pong.client.PongInterface}).
 * @author nuno
 */
public class Paddle {

	protected static final int VERTICAL = 1; // xx axis
	protected static final int HORIZONTAL = 2; // yy axis
	
	/**
	 * Left paddle
	 */
	public static final int LEFT = 0;
	
	/**
	 * Right paddle
	 */
	public static final int RIGHT = 1;
	
	/**
	 * Top paddle
	 */
	public static final int TOP = 2;
	
	/**
	 * Bottom paddle
	 */
	public static final int BOTTOM = 3;
	
	public static final int LAR = 50;
	public static final int ALT = 10;

	// vertical or horizontal
	private int move_type;
	// current position of the paddle
	private int currentX, currentY, width, height;

	// bounds of the field
	int x_field;
	int y_field;
	int w_field;
	int h_field;

	private boolean active;

	/**
	 * Class constructor. 
	 * @param type the type of the paddle (vertical or horizontal)
	 * @param side the side of the paddle (left, right, top or bottom)
	 * @param c the canvas that represents the field.
	 */
	public Paddle(int type, int side, Canvas c) {
		move_type = type;
		setActive(false);
		x_field = c.getX();
		y_field = c.getY();
		w_field = c.getWidth();
		h_field = c.getHeight();
		if(move_type==VERTICAL){
			if(side==LEFT){
				currentY = (h_field-x_field)/2 - (LAR/2);
				currentX = 10;								
			}
			else if(side==RIGHT){
				currentY = (h_field-x_field)/2 - (LAR/2);
				currentX = w_field-ALT-10;				
			}
			else throw new IllegalArgumentException("VERTICAL must be LEFT or RIGHT.");
			width = ALT;
			height = LAR;			
		}
		else if(move_type == HORIZONTAL){
			if(side==TOP){
				currentY = 10;
				currentX = (w_field/2) - (LAR/2);								
			}
			else if(side==BOTTOM){
				currentY = h_field-ALT-10;
				currentX = (w_field/2) - (LAR/2);				
			}
			else throw new IllegalArgumentException("HORIZONTAL must be TOP or BOTTOM.");
			width = LAR;
			height = ALT;
		}
		else throw new IllegalArgumentException("First arg must be HORIZONTAL or VERTICAL.");
	}

	/**
	 * Sets the paddle to active or inactive.
	 * @param true to set the paddle to active, false otherwise.
	 */
	protected void setActive(boolean active) {
		this.active = active;
	}

	/**
	 * returns true if the paddle is active, false otherwise.
	 * @return true if the paddle is active, false otherwise.
	 */
	protected boolean isActive() {
		return active;
	}

	/**
	 * Move the paddle to the given position and repaint it on the given graphics.
	 * @param p the new position of the paddle.
	 * @param g the graphics of the component where the paddle should be painted.
	 */
	public void move(Point p, Graphics g){
		clear(g);
		i_move(p.x,p.y);
		paint(g);
	}

	/**
	 * Returns the current xx position of the paddle.
	 * @return the current xx position of the paddle.
	 */
	public int getX() {
		return currentX;
	}

	/**
	 * Returns the current yy position of the paddle.
	 * @return the current yy position of the paddle.
	 */
	public int getY() {
		return currentY;
	}

	/**
	 * Returns the height of the paddle.
	 * @return the height of the paddle.
	 */
	public int getHeight() {
		return height;
	}

	/**
	 * Returns the width of the paddle.
	 * @return the width of the paddle.
	 */
	public int getWidth() {
		return width;
	}
	
	private void clear(Graphics g){
		g.clearRect(currentX,currentY,width,height);
	}
	
	private void i_move(int x, int y){
		if(move_type == VERTICAL){
			if(y+LAR > h_field)
				currentY = h_field-LAR;
			else
				currentY = y;
		}
		else{
			if(x+LAR > w_field)
				currentX = w_field-LAR;
			else
				currentX = x;
		}
	}
	
	/**
	 * Paint the paddle on the given Graphics.
	 * @param g the graphics of the component where to paint the paddle.
	 */
	protected void paint(Graphics g){
		Color oldColor = g.getColor();
		Color paintColor = null;
		if(active)
			paintColor = Color.DARK_GRAY;
		else
			paintColor = Color.RED;
		g.setColor(paintColor);
		g.fillRect(currentX,currentY,width,height);
		g.setColor(oldColor);
	}

}
