/*
 * Created on 15-Sep-2003
 *
 */
package pong.client;

import java.awt.Point;
import java.net.InetAddress;
import java.net.UnknownHostException;

import pong.BallState;
import pong.GameState;
import pong.events.MovePaddleEvent;
import pong.events.PlayerInfo;
import pong.events.RegisterFailedEvent;
import pong.events.RegisterPlayerEvent;
import pong.events.StartEvent;
import pong.events.StateEvent;
import appia.AppiaEventException;
import appia.Channel;
import appia.Direction;
import appia.Event;
import appia.Layer;
import appia.Session;
import appia.events.channel.ChannelInit;
import appia.message.ExtendedMessage;
import appia.protocols.common.InetWithPort;
import appia.protocols.common.RegisterSocketEvent;
import appia.protocols.tcpcomplete.TcpUndeliveredEvent;
import appia.xml.interfaces.InitializableSession;
import appia.xml.utils.SessionProperties;

/**
 * Client application. This application was builted as a protocol. It creates the Game environment
 * (to the player) and exchanges messages with the game server.
 * It requires reliable fifo at the bottom.
 * @author nuno
 */
public class PongClientSession extends Session implements InitializableSession {

	private Channel channel;
	private PongInterface pong;
	private InetWithPort local, server;
	private int myPaddle;
	private GameState gameState;

	/**
	 * Main class constructor.
	 * @param layer the corresponding layer.
	 */
	public PongClientSession(Layer layer) {
		super(layer);
	}

	/**
	 * Method used to initialize this session before creating the channel.
	 * @param remoteHost host name of the server.
	 * @param remotePort port number of the server.
	 */
	public void init(String remoteHost, int remotePort){
		try {
			this.server = new InetWithPort(InetAddress.getByName(remoteHost),remotePort);
		} catch (UnknownHostException e) {
			e.printStackTrace();
			server = null;
		}
	}

	/**
	 * Method invoked by Appia when a channel is created using XML.
	 * This session can be configured using XML with 2 parameters:
	 * "serverhost" - the server host name; "serverport" - the server port number.
	 * @param params the parameters and their values, from the XML configuration.
	 */
	public void init(SessionProperties params) {
		int sp = 0;
		String sh = null;
		if(params.containsKey("serverhost")){
			sh = params.getString("serverhost");
		}
		if(params.containsKey("serverport")){
			sp = params.getInt("serverport");
		}
		if(sp != 0 && sh != null)
			try {
				server = new InetWithPort(InetAddress.getByName(sh),sp);
			} catch (UnknownHostException e) {
				e.printStackTrace();
				server = null;
			}
	}

	/**
	 * Main event handler.
	 */
	public void handle(Event ev) {
		//System.out.println("PongClientSession Recebeu evento "+ev.getClass().getName());
		if (ev instanceof ChannelInit)
			handleChannelInit((ChannelInit) ev);
		else if (ev instanceof RegisterSocketEvent)
			handleRSE((RegisterSocketEvent) ev);
		else if (ev instanceof MovePaddleEvent)
			handleMouseMovedEvent((MovePaddleEvent) ev);
		else if (ev instanceof StartEvent)
			handleButtonStartEvent((StartEvent) ev);
		else if (ev instanceof PlayerInfo)
			handlePlayerInfo((PlayerInfo) ev);
		else if (ev instanceof StateEvent)
			handleStateEvent((StateEvent) ev);
		else if (ev instanceof RegisterFailedEvent){
			System.out.println("Registration in server "+server+" failed. Commiting suicide!");
			System.exit(0);		
		}
		else if (ev instanceof TcpUndeliveredEvent)
			handleTcpUndelivered((TcpUndeliveredEvent) ev);
		else
			// unexpected event. Forwarding it.
			try {
				ev.go();
			} catch (AppiaEventException e) {
				e.printStackTrace();
			}
	}

	private void handleTcpUndelivered(TcpUndeliveredEvent event) {
		/*
		 * Event received when the server dies, or the connection is lost
		 */
		
		System.out.println("Server on host "+event.who+" seems to be dead...");
		System.exit(2);
	}

	private void handleStateEvent(StateEvent event) {
		/*
		 * Event received from the server with a state update of the game.
		 */
		GameState gs = (GameState) event.getExtendedMessage().popObject();
		BallState bs = gs.getBallState();
		if (gameState == null){
			Point initPoint = new Point(bs.getXPosition(),bs.getYPosition());
			pong.createBall(initPoint);
			pong.setWindowTitle(myPaddle);
		}
		
		gameState = gs;
		pong.moveBall(bs);
		for (int i=0; i<GameState.MAX_PLAYERS; i++){
			pong.activePaddle(i,gs.getActivePlayers()[i]);
			pong.movePaddle(i,gs.getPlayerPosition(i));
			pong.updatePontuation(i,gs.getPlayerScore(i));
		}
	}

	private void handlePlayerInfo(PlayerInfo info) {
		/*
		 * event received from the server in response to a RegisterPlayerEvent.
		 * it contains the id of the player.
		 */
		ExtendedMessage msg = info.getExtendedMessage();
		myPaddle = msg.popInt();
		
		/*
		 * Graphic interface is created here.
		 */
		pong = new PongInterface(channel, myPaddle);		
	}

	private void handleRSE(RegisterSocketEvent event) {
		/*
		 * After the registration of the socket, this layer receives this
		 * event again, but know with usefull information:
		 * The socket and the localhost.
		 */
		if(!event.error)
			local = new InetWithPort(event.localHost,event.port);
		else {
			System.out.println("Error registering the Socket to connect to the game server.");
			System.exit(1);
		}
		
		/*
		 * Know that the socket is open, we can register in the game server.
		 */
		try {
			RegisterPlayerEvent rpe = new RegisterPlayerEvent(channel,Direction.DOWN,this);
			rpe.dest = server;
			rpe.go();
		} catch (AppiaEventException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @param init
	 */
	private void handleChannelInit(ChannelInit init) {
		/*
		 * This is the first event received by any session.
		 * It is also ALWAYS the first one that it sends.
		 * You should never send other event before forwarding the ChannelInit.
		 */
		
		channel = init.getChannel();
		try {
			init.go();
		} catch (AppiaEventException e) {
			e.printStackTrace();
		}
		/*
		 * Registers one socket on the bottom layer. you should do this before sending messages.
		 */
		try {
			RegisterSocketEvent rse = new RegisterSocketEvent(channel,Direction.DOWN,this);
			rse.go();
		} catch (AppiaEventException e1) {
			e1.printStackTrace();
		}
	}

	private void handleMouseMovedEvent(MovePaddleEvent event) {
		/*
		 * Event received by the interface, indicating a paddle move from the player.
		 */
		ExtendedMessage message = event.getExtendedMessage();
		if(event.getDir() == Direction.DOWN){
			/*
			 * these two fields are the source and destination od the message.
			 * in this case, from the client, to the server.
			 */
			event.source = local;
			event.dest = server;
			/*
			 * We need to put the Point that represents the new position of the paddle. 
			 */
			message.pushObject(event.getPoint());
			try {
				/*
				 * With events that enter the Appia channel using the asyncGo method,
				 * we need to initialize it with the source session and the init method.
				 * O atributo source do evento  diferente do metodo set/getSource()
				 */
				event.setSource(this);
				event.init();
				event.go();
			} catch (AppiaEventException e) {
				e.printStackTrace();
			}
		}
	}

	private void handleButtonStartEvent(StartEvent event) {
		/*
		 * received notification that the client pushed the "start" button.
		 * Sending it to the game server.
		 */
		if(event.getDir() == Direction.DOWN){
			event.source = local;
			event.dest = server;
			try {
				event.setSource(this);
				event.init();
				event.go();
			} catch (AppiaEventException e) {
				e.printStackTrace();
			}
		}
	}


}
