package pong.server;

import pong.BallState;

/**
 * This class represents the ball of the game. It is used by the game server to update
 * the ball position in the game area.
 * 
 * @author ncarvalho & jmocito
 */
public class Ball {

	// The state of the ball.
	private BallState ballState;

	// Value used to check if the ball hit the right wall.
	private int right_out = 390;
	// Value used to check if the ball hit the left wall.
	private int left_out = 10;
	// Value used to check if the ball hit the bottom wall.
	private int down_out = 290;
	// Value used to check if the ball hit the top wall.
	private int up_out = 10;

	// The radius of the ball
	final int radius = 10;
	
	// The paddles used by the players of the game.
	private Paddle[] paddles;

	/**
	 * Constructs a Ball object with the ball positioned at <tt>(x_field, y_field)</tt> in a field
	 * with width <tt>w</tt> and height <tt>h</tt>. It also associates the ball with the paddles
	 * used by the players of the game.
	 *  
	 * @param x_field the xx position of the ball in the game field. 
	 * @param y_field the yy position of the ball in the game field.
	 * @param w the width of the game field.
	 * @param h the height of the game field.
	 * @param paddles the paddles used by the players of the game.
	 */
	public Ball(int x_field, int y_field, int w, int h, Paddle[] paddles) {
		this.paddles = paddles;
		left_out = x_field;
		up_out = y_field;
		right_out = w - radius;
		down_out = h - radius;
		ballState = new BallState(200,300,8,6);
	}

	/**
	 * Moves the ball to the next position (computed from the current position and the 
	 * velocity vector) associated with the ball. If the ball hits a wall it's movement is
	 * reflected and an OutOfFieldException is thrown, containing the identifier of the wall
	 * that was hit.
	 * 
	 * @throws OutOfFieldException when the ball hits a wall.
	 */
	public void move() throws OutOfFieldException {
		ballState.incPosition();
		if (!isBallOnPaddles()) {
			int player = -1;
			if ((player = isBallOut()) != -1)
				throw new OutOfFieldException("Goooal!!!", player);
		}
	}

	/**
	 * Returns the current ball state.
	 * 
	 * @return the current ball state.
	 */
	protected BallState getBallState() {
		return ballState;
	}

	/**
	 * Sets the ball state.
	 * 
	 * @param ballState the new ball state.
	 */
	protected void setBallState(BallState ballState) {
		this.ballState = ballState;
	}

	/**
	 * Checks if the ball hit a paddle.
	 * 
	 * @return <tt>true</tt> if the ball hit a paddle, or <tt>false</tt> otherwise.
	 */
	private boolean isBallOnPaddles(){
		for (int i=0; i<paddles.length; i++)
			if( paddles[i].isBallOnPaddle(ballState) )
				return true;
		return false;
	}
	
	/**
	 * Checks if the ball hi a wall.
	 * 
	 * @return -1 if the ball did not hit a wall, or the wall identifier otherwise.
	 */
	private int isBallOut() {
		if (ballState.getXSpeed() < 0) {
			if (ballState.getXPosition() < left_out) {
				ballState.setXSpeed(-ballState.getXSpeed());
				return Paddle.LEFT;
			}
		} else if (ballState.getXSpeed() > 0) {
			if (ballState.getXPosition() > right_out) {
				ballState.setXSpeed(-ballState.getXSpeed());
				return Paddle.RIGHT;
			}
		}
		if (ballState.getYSpeed() < 0) {
			if (ballState.getYPosition() < up_out) {
				ballState.setYSpeed(-ballState.getYSpeed());
				return Paddle.TOP;
			}
		} else if (ballState.getYSpeed() > 0) {
			if (ballState.getYPosition() > down_out) {
				ballState.setYSpeed(-ballState.getYSpeed());
				return Paddle.BOTTOM;
			}
		}
		return -1;
	}
}