package pong.server;

import java.awt.Point;

import pong.BallState;
import pong.GameArea;

/**
 * This class represents a paddle of the game. It is used to compute new paddle positions
 * in response to players movements signaled to the server by a MovePaddleEvent.
 * 
 * @author ncarvalho & jmocito
 */
public class Paddle {

	/**
	 * Vertical movement identifier.
	 */ 
	public static final int VERTICAL = 1;
	
	/**
	 * Horizontal movement identifier.
	 */
	public static final int HORIZONTAL = 2;
	
	/**
	 * Identifier of the left paddle.
	 */
	public static final int LEFT = 0;
	
	/**
	 * Identifier of the right paddle.
	 */
	public static final int RIGHT = 1;
	
	/**
	 * Identifier of the top paddle.
	 */
	public static final int TOP = 2;
	
	/**
	 * Identifier of the bottom paddle.
	 */
	public static final int BOTTOM = 3;
	
	/**
	 * Width of the paddle.
	 */
	public static final int WIDTH = 50;
	
	/**
	 * Height of the paddle.
	 */
	public static final int HEIGHT = 10;

	// Type of movement (VERTICAL or HORIZONTAL)
	private int move_type;
	
	// Side of the paddle (LEFT or RIGHT if it is VERTICAL; TOP or BOTTOM if it is HORIZONTAL)
	private int side;
	// Current position of the paddle.
	private int currentX, currentY;
	// Size of the paddle.
	private int width, height;

	// Bounds of the field
	private int x_field;
	private int y_field;
	private int w_field;
	private int h_field;

	// States that the player is (or is not) active.
	private boolean active;

	/**
	 * Constructs a Paddle object of the specified type and at the specified side.
	 * 
	 * @param type the type of the paddle (HORIZONTAL or VERTICAL)
	 * @param side the side of the paddle (LEFT, RIGHT, TOP or BOTTOM)
	 * 
	 * @throws IllegalArgumentException if any of the parameters is no valid.
	 */
	public Paddle(int type, int side) {
		move_type = type;
		this.side = side;
		setActive(false);
		x_field = GameArea.X;
		y_field = GameArea.Y;
		w_field = GameArea.WIDTH;
		h_field = GameArea.HEIGHT;
		if(move_type==VERTICAL){
			if(side==LEFT){
				currentY = (h_field-x_field)/2 - (WIDTH/2);
				currentX = 10;								
			}
			else if(side==RIGHT){
				currentY = (h_field-x_field)/2 - (WIDTH/2);
				currentX = w_field-HEIGHT-10;				
			}
			else throw new IllegalArgumentException("VERTICAL must be LEFT or RIGHT.");
			width = HEIGHT;
			height = WIDTH;			
		}
		else if(move_type == HORIZONTAL){
			if(side==TOP){
				currentY = 10;
				currentX = (w_field/2) - (WIDTH/2);								
			}
			else if(side==BOTTOM){
				currentY = h_field-HEIGHT-10;
				currentX = (w_field/2) - (WIDTH/2);				
			}
			else throw new IllegalArgumentException("HORIZONTAL must be TOP or BOTTOM.");
			width = WIDTH;
			height = HEIGHT;
		}
		else throw new IllegalArgumentException("First arg must be HORIZONTAL or VERTICAL.");
	}

	/**
	 * Sets the active state of the paddle.
	 * 
	 * @param active the active state of the paddle.
	 */
	protected void setActive(boolean active) {
		this.active = active;
	}

	/**
	 * Checks if the paddle is active.
	 * 
	 * @return <tt>true</tt> if the player is active, or <tt>false</tt> otherwise.
	 */
	protected boolean isActive() {
		return active;
	}

	/**
	 * Moves the paddle to the specified position.
	 * 
	 * @param p the point specifying the new position.
	 */
	public void move(Point p){
		if(move_type == VERTICAL){
			if(p.y+WIDTH > h_field)
				currentY = h_field-WIDTH;
			else
				currentY = p.y;
		}
		else{
			if(p.x+WIDTH > w_field)
				currentX = w_field-WIDTH;
			else
				currentX = p.x;
		}
	}

	/**
	 * Returns the xx component of the paddle position.
	 * 
	 * @return the xx component of the paddle position.
	 */
	public int getX() {
		return currentX;
	}

	/**
	 * Returns the yy component of the paddle position.
	 * 
	 * @return the yy component of the paddle position.
	 */
	public int getY() {
		return currentY;
	}

	/**
	 * Returns the height of the paddle.
	 *  
	 * @return the height of the paddle.
	 */
	public int getHeight() {
		return height;
	}

	/**
	 * Returns the width of the paddle.
	 * 
	 * @return the width of the paddle.
	 */
	public int getWidth() {
		return width;
	}
	
	/**
	 * Checks if the ball collided with the paddle.
	 * 
	 * @param b the ball state containing its position.
	 * @return <tt>true</tt> if the ball collided with the paddle, or <tt>false</tt> otherwise.
	 */
	public boolean isBallOnPaddle(BallState b) {
		int right = x_field + currentX + width;
		int left = currentX - x_field;
		int top = currentY - y_field;
		int bottom =  y_field + currentY + height;
		int xPosition = b.getXPosition();
		int yPosition = b.getYPosition();
		int xSpeed = b.getXSpeed();
		int ySpeed = b.getYSpeed();
		 
		if (side == LEFT && xSpeed < 0 && yPosition > top && yPosition < bottom) {
			if (xPosition + xSpeed < right) {
				b.setXSpeed(-xSpeed);
				return true;
			}
		} else if (side == RIGHT && xSpeed > 0 && yPosition > top && yPosition < bottom) {
			if (xPosition + xSpeed > left) {
				b.setXSpeed(-xSpeed);
				return true;
			}
		}
		if (side == TOP && ySpeed < 0 && xPosition > left && xPosition < right) {
			if (yPosition + ySpeed < bottom) {
				b.setYSpeed(-ySpeed);
				return true;
			}
		} else if (side == BOTTOM && ySpeed > 0 && xPosition > left && xPosition < right) {
			if (yPosition + ySpeed > top) {
				b.setYSpeed(-ySpeed);
				return true;
			}
		}
		return false;
	}
}
