package pong.server;

import java.awt.Point;

import pong.GameArea;
import pong.GameState;
import pong.events.MoveBallTimer;
import pong.events.MovePaddleEvent;
import pong.events.PlayerInfo;
import pong.events.RegisterFailedEvent;
import pong.events.RegisterPlayerEvent;
import pong.events.StartEvent;
import pong.events.StateEvent;
import appia.AppiaEventException;
import appia.AppiaException;
import appia.Channel;
import appia.Direction;
import appia.Event;
import appia.EventQualifier;
import appia.Layer;
import appia.Session;
import appia.events.channel.ChannelInit;
import appia.protocols.common.InetWithPort;
import appia.protocols.common.RegisterSocketEvent;
import appia.protocols.tcpcomplete.TcpUndeliveredEvent;
import appia.xml.interfaces.InitializableSession;
import appia.xml.utils.SessionProperties;

/**
 * Appia session that implements the Pong server.
 * 
 * @author jmocito
 */
public class PongServerSession extends Session implements InitializableSession {

	// Maximum number of players
	private static final int MAX_PLAYERS = 4;
	// Ball timer timeout
	private static final int BALL_TIMER_TIMEOUT = 50;
	
	// Appia communication channel
	private Channel channel;
	// Local port
	private int port;
	
	// Current state of the game
	private GameState state;
	// List of paddles in the game (size = MAX_PLAYERS)
	private Paddle[] paddle = new Paddle[MAX_PLAYERS];
	// The ball of the game
	private Ball ball;
	// Number of players active in the game
	private int playerCount;
	// List of players active in the game
	private InetWithPort[] players = new InetWithPort[MAX_PLAYERS];
	
	/**
	 * Constructs a PongServerSession object.
	 * 
	 * @param layer the PongServerLayer instance associated with this session.
	 */
	public PongServerSession(Layer layer) {
		super(layer);
		paddle[0] = new Paddle(Paddle.VERTICAL, Paddle.LEFT);
		paddle[1] = new Paddle(Paddle.VERTICAL, Paddle.RIGHT);
		paddle[2] = new Paddle(Paddle.HORIZONTAL, Paddle.TOP);
		paddle[3] = new Paddle(Paddle.HORIZONTAL, Paddle.BOTTOM);
		ball = new Ball(GameArea.X, GameArea.Y, GameArea.WIDTH, GameArea.WIDTH, paddle);
		state = new GameState(ball.getBallState());
	}

	/**
	 * Initialization method used by Appia XML support to fill the parameters
	 * used in this session.
	 */
	public void init(SessionProperties params) {
		if (params.containsKey("port"))
			port = params.getInt("port");
	}
	
	/**
	 * Generic handle for all events accepted by this session.
	 */
	public void handle(Event event) {
		if (event instanceof ChannelInit)
			handleChannelInit((ChannelInit) event);
		else if (event instanceof RegisterSocketEvent)
			handleRegisterSocketEvent((RegisterSocketEvent) event);
		else if (event instanceof RegisterPlayerEvent)
			handleRegisterPlayerEvent((RegisterPlayerEvent) event);
		else if (event instanceof StartEvent)
			handleStartEvent((StartEvent) event);
		else if (event instanceof MovePaddleEvent)
			handleMovePaddleEvent((MovePaddleEvent) event);
		else if (event instanceof MoveBallTimer)
			handleMoveBallTimer((MoveBallTimer) event);
		else if (event instanceof TcpUndeliveredEvent)
			handleUndeliveredEvent((TcpUndeliveredEvent) event);
		else
			try {
				event.go();
			} catch (AppiaEventException e) {
				e.printStackTrace();
			}
	}

	/**
	 * Handler for the ChannelInit event.
	 * 
	 * This event is received only once when the channel is created. It tries to register
	 * the socket in the underlying execution environment (OS) and starts the the periodic
	 * timer that drives the ball movement.
	 * 
	 * @param init the ChannelInit event received in this session.
	 */
	private void handleChannelInit(ChannelInit init) {
		channel = init.getChannel();
		try {
			init.go();
		} catch (AppiaEventException e1) {
			e1.printStackTrace();
		}
		try {
			RegisterSocketEvent rse = new RegisterSocketEvent(channel,Direction.DOWN,this,port);
			rse.go();
		} catch (AppiaEventException e) {
			e.printStackTrace();
		}
		try {
			MoveBallTimer timer = new MoveBallTimer(BALL_TIMER_TIMEOUT,channel,this,EventQualifier.ON);
			timer.go();
		} catch (AppiaEventException e) {
			e.printStackTrace();
		} catch (AppiaException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Handler for the RegisterSocketEvent event.
	 * 
	 * This event only certifies that the chosen local port is not already in use.
	 * 
	 * @param event the RegisterSocketEvent event received in this session.
	 */
	private void handleRegisterSocketEvent(RegisterSocketEvent event) {
		if (event.error) {
			System.err.println("Port "+event.port+" is already in use. Please choose a different port.");
			System.exit(1);
		}
	}
	
	/**
	 * Handler for the RegisterPlayerEvent event.
	 * 
	 * This event is received whenever a player wants to join the game. This method
	 * checks if the player can join the game (playercount < MAX_PLAYERS) and issues a
	 * PlayerInfo event followed by a StateEvent event in the affirmative case, or a
	 * RegisterFailedEvent event in the negative case. 
	 * 
	 * @param event the RegisterPlayerEvent event received in this session.
	 */
	private void handleRegisterPlayerEvent(RegisterPlayerEvent event) {
		if (playerCount < MAX_PLAYERS) {
			int playerNumber = getFreePlayer();
			players[playerNumber] = (InetWithPort) event.source;
			try {
				PlayerInfo pi = new PlayerInfo(channel, Direction.DOWN, this);
				//pi.getExtendedMessage().pushInt(players.indexOf(event.source));
				pi.getExtendedMessage().pushInt(playerNumber);
				pi.dest = event.source;
				pi.go();
			} catch (AppiaEventException e) {
				//players.remove(event.source);
				players[playerNumber] = null;
				e.printStackTrace();
			}
			playerCount++;
			try {
				StateEvent se = new StateEvent(channel, Direction.DOWN, this);
				se.getExtendedMessage().pushObject(state);
				se.dest = event.source;
				se.go();
			} catch (AppiaEventException e) {
				e.printStackTrace();
			}
		}
		else {
			RegisterFailedEvent rfe;
			try {
				rfe = new RegisterFailedEvent(channel, Direction.DOWN, this);
				rfe.dest = event.source;
				rfe.go();
			} catch (AppiaEventException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * Handler for the StartEvent event.
	 * 
	 * This event is received whenever a player asserts that he's ready to start
	 * playing the game. The game state is updated to certify this player as an active
	 * player and is broadcasted to all players.
	 * 
	 * @param event the StartEvent event received in this session.
	 */
	private void handleStartEvent(StartEvent event) {
		int playerNumber = getPlayerNumber((InetWithPort) event.source);
		if (!state.getActivePlayers()[playerNumber]) {
			state.setActivePlayer(playerNumber);
			broadcastState();
		}
	}

	/**
	 * Handler for the MovePaddleEvent event.
	 * 
	 * This event is received whenever a player tries to move his paddle. The new position
	 * of the paddle is computed and updated in the game state. The state is then broadcasted
	 * to all players.
	 * 
	 * @param event the MovePaddleEvent event received in this session.
	 */
	private void handleMovePaddleEvent(MovePaddleEvent event) {
		Point p = (Point) event.getExtendedMessage().popObject();
		int playerNumber = getPlayerNumber((InetWithPort) event.source);
		state.setPlayerPosition(playerNumber,p);
		paddle[playerNumber].move(p);
		broadcastState();
	}

	/**
	 * Handler for the MoveBallTimer timer.
	 * 
	 * This event is received whenever the periodic timer associated with the ball movement
	 * expires. The position of the ball is updated and the new game state is broadcasted
	 * to all players.
	 * 
	 * @param timer the MoveBallTimer timer received in this session.
	 */
	private void handleMoveBallTimer(MoveBallTimer timer) {
		try {
			ball.move();
		} catch (OutOfFieldException e) {
			state.incPlayerScore(e.getWall());
		}
		broadcastState();
	}
	
	/**
	 * Handler for the UndeliveredEvent event.
	 * 
	 * This event is received whenever an event is sent to some client that is not there anymore.
	 * It may be because it voluntarily left, or because it crashed. Either way, the player is
	 * removed from the active player list and its position and points are reset to their initial
	 * values. 
	 * 
	 * @param event the UndeliveredEvent event received in this session.
	 */
	public void handleUndeliveredEvent(TcpUndeliveredEvent event) {
		int playerNumber = getPlayerNumber((InetWithPort) event.who);
		if (playerNumber != -1) {
			state.resetPlayer(playerNumber);
			players[playerNumber] = null;
			playerCount--;
			broadcastState();
		}
	}
	
	/**
	 * Broadcasts the game state to all the players registered in the game.
	 */
	private void broadcastState() {
		for (int i = 0; i < players.length; i++) {
			if (players[i] != null) {
				try {
					StateEvent se = new StateEvent(channel, Direction.DOWN, this);
					se.getExtendedMessage().pushObject(state);
					se.dest = players[i];
					se.go();
				} catch (AppiaEventException e) {
					e.printStackTrace();
				}
			}
		}
	}
	
	private int getFreePlayer() {
		for (int i = 0; i < players.length; i++)
			if (players[i] == null)
				return i;
		return -1;
	}
	
	private int getPlayerNumber(InetWithPort iwp) {
		for (int i = 0; i < players.length; i++)
			if (players[i] != null && players[i].equals(iwp))
				return i;
		return -1;
	}
}
